﻿using System;
using System.Collections.Specialized;
using System.IO;
using System.Net;
using System.ServiceModel.Web;
using System.Configuration;

namespace CloudMap.Contracts
{
    public class WMSProxyService : IWMSProxyService
    {
        public Stream WMSRequest(string map)
        {
#if DEBUG
            Console.WriteLine("Proxy call!");
#endif
            var wms = GetWMSbyMap(map);
            return RelayUrl(string.Format("{0}{1}{2}", 
                                          wms, 
                                          (wms.Contains("?") ? "&" : "?"),
                                          WebOperationContext.Current.IncomingRequest.UriTemplateMatch.QueryParameters));                 
        }

        private string GetWMSbyMap( string map )
        {
            var services = ConfigurationManager.GetSection("WMSServices") 
                           as NameValueCollection;
            return services != null ? services[map] : null;
        }

        private Stream RelayUrl(string urlToLoadFrom)
        {
            var webRequest = HttpWebRequest.Create(urlToLoadFrom) as HttpWebRequest;
            // Important! Keeps the request from blocking after the first
            // time!
            webRequest.KeepAlive = false;
            webRequest.Credentials = CredentialCache.DefaultCredentials;
            using (var backendResponse = (HttpWebResponse)webRequest.GetResponse())
            {
                using (var receiveStream = backendResponse.GetResponseStream())
                {
                    var ms = new MemoryStream();
                    var response = WebOperationContext.Current.OutgoingResponse;
                    // Copy headers                    
                    // Check if header contains a contenth-lenght since IE
                    // goes bananas if this is missing
                    bool contentLenghtFound = false;
                    foreach (string header in backendResponse.Headers)
                    {
                        if (string.Compare(header, "CONTENT-LENGTH", true) == 0)
                        {
                            contentLenghtFound = true;
                        }
                        response.Headers.Add(header, backendResponse.Headers[header]);
                    }
                    // Copy contents                    
                    var buff = new byte[1024];
                    var length = 0;
                    int bytes;
                    while ((bytes = receiveStream.Read(buff, 0, 1024)) > 0)
                    {
                        length += bytes;
                        ms.Write(buff, 0, bytes);
                    }
                    // Add contentlength if it is missing
                    if (!contentLenghtFound) response.ContentLength = length;
                    // Set the stream to the start
                    ms.Position = 0;
                    return ms;
                }
            }   
        }
    }
}